@@ -1,15 +1,15 @@
-# 
+#
 # This file is part of Config-Model
-# 
+#
 # This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
-# 
+#
 # This is free software, licensed under:
-# 
+#
 #   The GNU Lesser General Public License, Version 2.1, February 1999
-# 
+#
 package Config::Model;
 BEGIN {
-  $Config::Model::VERSION = '1.218';
+  $Config::Model::VERSION = '1.219';
 }
 require Exporter;
 use Carp;
@@ -42,10 +42,12 @@ Config::Model - Create tools to validate, migrate and edit configuration files
 
 =head1 VERSION
 
-version 1.218
+version 1.219
 
 =head1 SYNOPSIS
-  
+
+=head2 Perl program
+
  use Config::Model;
  use Log::Log4perl qw(:easy) ;
  Log::Log4perl->easy_init($WARN);
@@ -78,6 +80,31 @@ version 1.218
 
  # now look for new mini.ini file un current directory
 
+=head2 More convenient
+
+ $ mkdir -p lib/Config/Model/models/
+ $ echo "[ { name => 'MiniModel',
+             element => [ [qw/foo bar baz/ ] => { type => 'leaf', value_type => 'uniline' }, ],
+             read_config => { backend => 'IniFile', auto_create => 1,
+                              config_dir => '.', file => 'mini.ini',
+                            }
+           }
+         ] ; " > lib/Config/Model/models/MiniModel.pl
+ $ config-edit -model MiniModel -model_dir lib/Config/Model/models/ -ui none bar=BARV foo=FOOV baz=BAZV
+ $ cat mini.ini
+
+=head2 Look Ma, no Perl
+
+ $ echo "Make sure that Config::Model::Itself is installed"
+ $ mkdir -p lib/Config/Model/models/
+ $ config-model-edit -model MiniModel -save \
+   class:MiniModel element:foo type=leaf value_type=uniline - \
+   element:bar type=leaf value_type=uniline - \
+   element:baz type=leaf value_type=uniline - \
+   read_config:0 backend=IniFile file=mini.ini config_dir=. auto_create=1 - - -
+ $ config-edit -model MiniModel -model_dir lib/Config/Model/models/ -ui none bar=BARV foo=FOOV baz=BAZV
+ $ cat mini.ini
+
 =head1 DESCRIPTION
 
 Config::Model enables a project developer to provide an interactive
@@ -137,13 +164,13 @@ will be made of 3 parts :
 
 =over
 
-=item 1
+=item 1.
 
 A reader and writer that will parse the configuration file and transform
 in a tree representation within Config::Model. The values contained in this
 configuration tree can be written back in the configuration file(s).
 
-=item 2
+=item 2.
 
 A validation engine which is in charge of validating the content and
 structure of configuration stored in the configuration tree. This
@@ -151,7 +178,7 @@ validation engine will follow the structure and constraint declared in
 a configuration model. This model is a kind of schema for the
 configuration tree.
 
-=item 3
+=item 3.
 
 A user interface to modify the content of the configuration tree. A
 modification will be validated instantly by the validation engine.
@@ -207,7 +234,7 @@ enum like type, default value ...)
 
 The targeted audience (beginer, advanced, master)
 
--=item *
+=item *
 
 The on-line help
 
@@ -448,7 +475,7 @@ classes). But they must be declared as a DAG (directed acyclic graph).
 
 Each configuration class declaration specifies:
 
-=over 8
+=over
 
 =item *
 
@@ -472,7 +499,7 @@ Each element will specify:
 
 =over
 
-=item 8
+=item *
 
 Most importantly, the type of the element (mostly C<leaf>, or C<node>)
 
@@ -1857,6 +1884,43 @@ sub list_one_class_element {
     return $res ;
 }
 
+=head1 Available models
+
+=cut
+
+sub available_models {
+   
+    my $path = $INC{"Config/Model.pm"} ;
+    $path =~ s/\.pm// ;
+    my (%categories, %model_info, %models ) ;
+
+    get_logger("Model")->trace("available_models: path is $path");
+    foreach my $dir (glob("$path/*.d")) {
+        my ($cat) = ( $dir =~ m!.*/([\w\-]+)\.d! );
+        
+        get_logger("Model")->trace("available_models: category dir $dir");
+        
+        foreach my $file (sort glob("$dir/*")) {
+            next if $file =~ m!/README! ;
+            my ($name) = ($file =~ m!.*/([\w\-]+)! );
+            get_logger("Model")->debug("available_models: opening file $file");
+            open (F, $file) || die "Can't open file $file:$!" ;
+            while (<F>) {
+                chomp ;
+                s/^\s+// ;
+                s/\s+$// ;
+                s/#.*// ;
+                my ($k,$v) = split /\s*=\s*/ ;
+                next unless $v ;
+                push @{$categories{$cat}} , $name if $k =~ /model/i;
+                $model_info{$name}{$k} = $v ; 
+                $models{$name} = $v if $k =~ /model/i; 
+            }
+        }
+    }
+    return \%categories, \%model_info, \%models ;
+}
+
 =head1 Error handling
 
 Errors are handled with an exception mechanism (See
@@ -1916,7 +1980,7 @@ Dominique Dumont, (ddumont at cpan dot org)
 
 L<Config::Model::Instance>,
 
-http://sourceforge.net/apps/mediawiki/config-model/index.php?title=Creating_a_model
+L<http://sourceforge.net/apps/mediawiki/config-model/index.php?title=Creating_a_model>
 
 =head2 Model elements
 
